/*!
 * ehsmsgqueue
 * Copyright(c) 2018-2022 End House Software
 * MIT Licensed
 * 
 * Version 1.0.0    Initial version
 * Version 1.0.1    Bug fixes
 * Version 1.0.2    Now uses Mongo database
 * Version 1.0.3    Corrections to initMsgQueues and getMsg functions
 * Version 1.0.4    Use of process.netxTick in getMsg
 * Version 1.0.5    Removed msgQueueCleanupProcess function and msgPoll just logs message to console
 * Version 1.0.6    currentMsgId is now per user
 * Version 1.0.7 + 1.0.8   Updated README.md file
 * 
 */

'use strict'

//var mysql = require('mysql');
var MongoClient = require('mongodb').MongoClient;

var _ = require('lodash');
var $ = require('jquery');

var uri = "mongodb+srv://username:password@ehs0-ncww2.mongodb.net/dbname";
var msgPollPeriod = 5000;
var bMsgQueuePollActive = false;
var msgPoll = null;
var currentMsgId = new Object;
var msgQueuePolled = "";

//var mysqlcon = mysql.createConnection({
//  host: "localhost",
//  user: "xxx",
//  password: "xxx",
//  database: "gavin_endhousesoftware"
//});

//mysqlcon.connect(function(err) {
//  if (err) throw err;
//  console.log('Connected to MySQL database.');
//});
	
    const versionString = "ehsmsgqueue - Copyright(c) 2018-2022 End House Software - Version 1.0.8";
    function getVersionString() {return versionString;}

    function initMsgQueues(msgQueue,username,callback) {
        MongoClient.connect(uri, function(err, db) {
            if (err) {
              console.log("Remote Mongo Database(ehsmsgqueue:initMsgQueues): Not Connected");
              throw err;
            } 
            console.log("Remote Mongo Database(ehsmsgqueue:initMsgQueues): Connected");
            var dbo = db.db("endhousesoftware");
	        var query = { queuename: msgQueue };
            dbo.collection("msgqueues").find(query).toArray(function(err, result) {
                if (err) throw err;
                currentMsgId[username] = result.length;
                console.log("Set currentMsgId(init):" + currentMsgId[username]);
                db.close();
            });
        });

    //mysqlcon.query("SELECT MAX(ehsmsgserverID) AS val FROM ehsmsgservers",function(err,results,fields) {
    //    if (err) throw err;
    //
    //    currentMsgId = parseInt(results[0].val);
    // });    
    callback(null,0);
    }

function getMsg(msgQueue,username,callback) {
    var msg = "";
    
    console.log("getMsg:" + username);

    MongoClient.connect(uri, function(err, db) {
        if (err) {
          console.log("Remote Mongo Database(getMsg): Not Connected");
          throw err;
        } 
        //console.log("Remote Mongo Database(getMsg): Connected");
        var dbo = db.db("endhousesoftware");
        var query = { queuename: msgQueue };
        dbo.collection("msgqueues").find(query).toArray(function(err, result) {
            if (err) throw err;

            if (result.length > currentMsgId[username]) {
                msg = result[currentMsgId[username]].queuemsg;
                currentMsgId[username] = currentMsgId[username] + 1;
                //console.log("Message(getMsg):"+msg);
                process.nextTick(callback,null,msg);
           } else {
                //console.log("Message(getMsg):"+msg);
                process.nextTick(callback,null,msg);            
            }
            db.close();
        });
    });

    //mysqlcon.query("SELECT ehsmsgserverID,ehsmsgserverMsg FROM ehsmsgservers WHERE ehsmsgserverQueue='" + msgQueue + "' AND ehsmsgserverID>" + currentMsgId.toString() +" LIMIT 1",function(err,results,fields) {
    //    if (err) throw err;
    //
    //    if (results.length == 0) {
    //    } else {
    //        currentMsgId = parseInt(results[0].ehsmsgserverID);
    //        var msg = results[0].ehsmsgserverMsg;    
    //    }
    // });
}

function setMsg(msgQueue,msg,callback) {
    MongoClient.connect(uri, function(err, db) {
        if (err) {
          console.log("Remote Mongo Database(setMsg): Not Connected");
          throw err;
        } 
        //console.log("Remote Mongo Database(setMsg): Connected");
        var dbo = db.db("endhousesoftware");
        var myobj = {queuename: msgQueue,queuemsg: msg};
        dbo.collection("msgqueues").insertOne(myobj, function(err, res) {
            if (err) throw err;
            db.close();
          });
    });   

   // mysqlcon.query("INSERT INTO ehsmsgservers (ehsmsgserverID,ehsmsgserverQueue,ehsmsgserverMsg) VALUES (null,'" + msgQueue + "','" + msg + "')",function(err,results,fields) {
   //     if (err) throw err;
   // });

   callback(null,0);
}

function startMsgQueuePoll(msgQueue,callback) {
    msgQueuePolled = msgQueue;
    msgPoll = setInterval(function() {
        console.log("msg queue poll");
    },msgPollPeriod);
    bMsgQueuePollActive = true;

    callback(null,0);
}

function stopMsgQueuePoll(callback) {
    if (bMsgQueuePollActive) {
        clearInterval(msgPoll);
        bMsgQueuePollActive = false;
        msgQueuePolled = "";

        callback(null,0);
    }
}

function isMsgQueuePollActve(callback) {
    callback(null,bMsgQueuePollActive);
}

function setMsgPollPeriod(period,callback) {
    msgPollPeriod = period;

    callback(null,0);
}

module.exports = {
    getVersionString,
    initMsgQueues,
    getMsg,
    setMsg,
    startMsgQueuePoll,
    stopMsgQueuePoll,
    isMsgQueuePollActve,
    setMsgPollPeriod
 };
